==> fso-metadata-fork-v_0.5.tar.bz2: fso-metadata-fork-v_0.5/fso_metadata/api_call.py <==
import itertools
from typing import Type, Union

import pandas as pd

from fso_metadata.api_class import Api
from fso_metadata.constants import DCAT_URL
from fso_metadata.utils import stringify_filters


###############################################################
########################## i14y APIs ##########################
###############################################################

def get_codelist(
    identifier: str, 
    export_format: str="SDMX-ML", 
    version_format: float=2.1, 
    annotations: bool=False
) -> Union[dict, pd.DataFrame]:
    '''
    Get a codelist based on an identifier
    Parameters:
        - identifier (str): the codelist's identifier
        - export_format (str, default="SDMX-ML"): the export's format
           Available are CSV, XLSX, SDMX-ML or SDMX-JSON.
        - version_format (float, default=2.1): the export format's version 
          (2.0 or 2.1 when format is SDMX-ML).
        - annotations (bool, default=False): flag to include annotations
        
    Returns:
        - response (pd.DataFrame or dict) based on the export format
            - a pd.DataFrame if export_format was CSV or XLSX
            - a dictionnary if export_format was SDMX-ML or SDMX-JSON.
    '''
    api = Api(
        api_type = 'codelist',
        _id = identifier,
        export_format = export_format, 
        version_format = version_format,
        parameters = f'annotations={str(annotations).lower()}'
    )
    return api.get_response()


def get_content_configuration() -> dict:
    '''
    Get the display information for the available configured content
    Returns:
        - response (dict): the configured content's display information
    '''
    api = Api(api_type = 'content_configuration')
    return api.get_response()


def get_identifier_content(identifier) -> dict:
    '''
    Get a nomenclature information based on its identifier
    Parameters:
        - identifier (str): the nomenclature's identifier
    Returns:
        - response (dict): the nomenclature's information
    '''
    api = Api(api_type = 'content_configuration_identifier', _id = identifier)
    return api.get_response()


def get_dataset_description(identifier: str, language: str = 'fr') -> dict:
    '''
    Get the dcat dataset description
    Parameters:
        - identifier (str): the dataset's identifier
        - language (str, default='fr'): the language of the response data
            Available are 'fr', 'de', 'it', 'en'.
    Returns:
        - response: description's dictionnary
    '''
    api = Api(
        api_type = 'dcat_dataset_description', 
        _id = identifier, 
        language = language
    )
    return api.get_response()


def get_dataset_information(identifier: str, language: str = 'fr') -> dict:
    '''
    Get the dcat dataset information
    Parameters:
        - identifier (str): the dataset's identifier
        - language (str, default='fr'): the language of the response data
            Available are 'fr', 'de', 'it', 'en'.
    Returns:
        - response: information's dictionnary
    '''
    api = Api(
        api_type = 'dcat_dataset_information', 
        _id = identifier, 
        language = language
    )
    return api.get_response()


def get_data_structure(identifier: str, language: str = 'fr') -> dict:
    '''
    Get the data structure
    Parameters:
        - identifier (str): the dataset's identifier
        - language (str, default='fr'): the language of the response data
            Available are 'fr', 'de', 'it', 'en'.
    Returns:
        - response: datastructure dictionnary
    '''
    api = Api(api_type = 'dcat_data_structure', _id = identifier, language = language)
    return api.get_response()


def get_nomenclature_path_nodes(
    identifier: str, 
    path: str, 
    filters: dict, 
    language: str = 'fr'
) -> dict:
    '''
    Get the nodes of a path within a nomenclature
    Parameters:
        - identifier (str): the nomenclature's identifier
        - path (str): the path leading to the nodes
        - filters (dict, default={}): the filters to apply
        - language (str, default='fr'): the language of the response data
            Available are 'fr', 'de', 'it', 'en'.
    Returns:
        - response: dictionnary of the nodes
    '''
    api = Api(
        api_type = 'nomenclature_path_nodes', 
        _id = identifier, 
        language = language, 
        parameters = filters
    )
    return api.get_response()


def get_nomenclature_one_level(
    identifier: str, 
    level_number: int, 
    filters: dict, 
    language: str = 'fr', 
    annotations: bool = False
) -> pd.DataFrame:
    '''
    Get one level of a nomenclature
    Parameters:
        - identifier (str): nomenclature's identifier
        - level_number (int): level to export
        - filter (default={}): additionnal filters
        - language (str, default='fr'): response data's language
            Available are 'fr', 'de', 'it', 'en'.
        - annotations (bool, default=False): flag to include annotations
    Returns:
        - response (pd.DataFrame): dataframe with 3 columns 
        (Code, Parent and Name in the selected language)
    '''
    parameters = (
        f'language={language}'
        f'&level={level_number}'
        f'&annotations={str(annotations).lower()}'
        f'&{stringify_filters(filters)}'
    )
    api = Api(
        api_type = 'nomenclature_one_level', 
        _id = identifier, 
        parameters = parameters,
        export_format='CSV'
    )
    return api.get_response()


def get_nomenclature_multiple_levels(
    identifier: str, 
    level_from: int, 
    level_to: int, 
    filters: dict, 
    language: str = 'fr', 
    annotations: bool = False
) -> pd.DataFrame:
    '''
    Get multiple levels of a nomenclature (from `level_from` to `level_to`)
    Parameters:
        - identifier (str): nomenclature's identifier
        - level_from (int): the 1st level to include
        - level_to (int): the last level to include
        - filter (default={}): additionnal filters
        - language (str, default='fr'): response data's language
            Available are 'fr', 'de', 'it', 'en'.
        - annotations (bool, default=False): flag to include annotations
    Returns:
        - response (pd.DataFrame): dataframe columns 
        from `level_from` to `level_to` codes
    '''
    # Call api with appropriate url and parameters
    parameters = (
        f'language={language}'
        f'&levelFrom={level_from}'
        f'&levelTo={level_to}'
        f'&annotations={str(annotations).lower()}'
        f'&{stringify_filters(filters)}'
    )
    api = Api(
        api_type = 'nomenclature_multiple_levels', 
        _id = identifier, 
        parameters = parameters,
        export_format='CSV'
    )
    df = api.get_response()
    
    # Post-processing: 
    # fill sub groups rows with parent group's values (instead of NaN)
    group_columns = [*itertools.takewhile(lambda col: col != 'Code', df.columns)]
    df[group_columns[0]] = df[group_columns[0]].fillna(method="ffill")
    df[group_columns[1:]] = (
        df.groupby([group_columns[0]])[group_columns[1:]]
    ).apply(lambda x: x.ffill())
        
    return df


def query_nomenclature(
    identifier: str, 
    query: str, 
    page_number: int, 
    page_size: int, 
    filters: dict, 
    language: str
) -> dict:
    '''
    Search query within a nomenclature
    Parameters:
        - identifier (str): the nomenclature's identifier
        - query (str): the search query
        - page_number (int): the number of the result page to return
        - page_size (int): the size of each page result
        - filter (default={}): additionnal filters
        - language (str, default='fr'): response data's language 
            Available are 'fr', 'de', 'it', 'en'.
    Returns:
        - response (dict): the query result
    '''
    parameters = {
        'language': language, 
        'query': query, 
        'page': page_number, 
        'pageSize': page_size
    }
    parameters = {**parameters, **filters}
    
    api = Api(
        api_type = 'nomenclature_search', 
        _id = identifier, 
        parameters = parameters
    )
    
    return api.get_response()


###############################################################
########################## dcat APIs ##########################
###############################################################

def dcat_list_all_agents():
    '''
    List all agents
    Returns:
        - response (dict): all agents
    '''
    api = Api(api_type = 'agents_list', root_url = DCAT_URL)
    return api.get_response()


def dcat_get_agent_from_id(agent_id):
    '''
    Get the agent with the corresponding agent id
    Parameters:
        - agent_id (str): agent's id
    Returns:
        - response (dict): agent with this id
    '''
    api = Api(api_type = 'agent_id', _id = agent_id, root_url = DCAT_URL)
    return api.get_response()


def dcat_list_all_datasets():
    '''
    List all datasets
    Returns:
        - response (dict): list of all datasets 
    '''
    api = Api(api_type = 'dataset_list', root_url=DCAT_URL)
    return api.get_response()


def dcat_get_distributions_from_dataset_id(dataset_id: str):
    '''
    Get all distributions for the dataset with the corresponding dataset id
    Parameters:
        - dataset_id (str): dataset's id
    Returns:
        - response (dict): distributions for the dataset with dataset's id
    '''
    api = Api(
        api_type = 'dataset_id_distributions', 
        _id = dataset_id, 
        root_url = DCAT_URL
    )
    return api.get_response()


def dcat_get_dataset_from_id(dataset_id: str):
    '''
    Get the dataset with the corresponding id
    Parameters:
        - dataset_id (str): dataset's id
    Returns:
        - response (dict): the dataset
    '''
    api = Api(api_type = 'dataset_id', _id = dataset_id, root_url = DCAT_URL)
    return api.get_response()


def dcat_get_dataset_from_identifier(identifier: str):
    '''
    Get the dataset with the corresponding identifier
    Parameters:
        - identidier (str): dataset's identifier
    Returns:
        - response (dict): the dataset
    '''
    api = Api(api_type = 'dataset_identifier', _id = identifier, root_url=DCAT_URL)
    return api.get_response()


def dcat_get_distributions_from_dataset_identifier(identifier: str):
    '''
    Get all distributions for the dataset with the corresponding identifier.
    Parameters:
        - identidier (str): dataset's identifier
    Returns:
        - response (dict): all distributions for the dataset with 
        the corresponding dataset identifier
    '''
    api = Api(
        api_type = 'dataset_identifier_distributions', 
        _id = identifier, 
        root_url = DCAT_URL
    )
    return api.get_response()


def dcat_list_all_distributions():
    '''
    List all distributions
    Returns:
        - response (dict): all distributions
    '''
    api = Api(api_type = 'distributions_list', root_url = DCAT_URL)
    return api.get_response()


def dcat_get_distribution_from_id(distribution_id: str):
    '''
    Get the distribution with the corresponding id
    Parameters:
        - distribution_id (str): distribution's id
    Returns:
        - response (dict): the distribution
    '''
    api = Api(
        api_type = 'distribution_id', 
        _id = distribution_id, 
        root_url = DCAT_URL
    )
    return api.get_response()

==> fso-metadata-fork-v_0.6.tar.bz2: fso-metadata-fork-v_0.6/fso_metadata/api_call.py <==
import itertools
from typing import Union

import pandas as pd

from fso_metadata.api_class import Api
from fso_metadata.constants import DCAT_URL
from fso_metadata.utils import stringify_filters


###############################################################
# i14y APIs
###############################################################


def get_codelist(
    identifier: str,
    export_format: str = "SDMX-ML",
    version_format: float = 2.1,
    annotations: bool = False,
) -> Union[dict, pd.DataFrame]:
    """
    Get a codelist based on an identifier
    Parameters:
        - identifier (str): the codelist's identifier
        - export_format (str, default="SDMX-ML"): the export's format
           Available are CSV, XLSX, SDMX-ML or SDMX-JSON.
        - version_format (float, default=2.1): the export format's version
          (2.0 or 2.1 when format is SDMX-ML).
        - annotations (bool, default=False): flag to include annotations

    Returns:
        - response (pd.DataFrame or dict) based on the export format
            - a pd.DataFrame if export_format was CSV or XLSX
            - a dictionnary if export_format was SDMX-ML or SDMX-JSON.
    """
    api = Api(
        api_type="codelist",
        _id=identifier,
        export_format=export_format,
        version_format=version_format,
        parameters=f"annotations={str(annotations).lower()}",
    )
    return api.get_response()


def get_content_configuration() -> dict:
    """
    Get the display information for the available configured content
    Returns:
        - response (dict): the configured content's display information
    """
    api = Api(api_type="content_configuration")
    return api.get_response()


def get_identifier_content(identifier) -> dict:
    """
    Get a nomenclature information based on its identifier
    Parameters:
        - identifier (str): the nomenclature's identifier
    Returns:
        - response (dict): the nomenclature's information
    """
    api = Api(api_type="content_configuration_identifier", _id=identifier)
    return api.get_response()


def get_dataset_description(identifier: str, language: str = "fr") -> dict:
    """
    Get the dcat dataset description
    Parameters:
        - identifier (str): the dataset's identifier
        - language (str, default='fr'): the language of the response data
            Available are 'fr', 'de', 'it', 'en'.
    Returns:
        - response: description's dictionnary
    """
    api = Api(
        api_type="dcat_dataset_description", _id=identifier, language=language
    )
    return api.get_response()


def get_dataset_information(identifier: str, language: str = "fr") -> dict:
    """
    Get the dcat dataset information
    Parameters:
        - identifier (str): the dataset's identifier
        - language (str, default='fr'): the language of the response data
            Available are 'fr', 'de', 'it', 'en'.
    Returns:
        - response: information's dictionnary
    """
    api = Api(
        api_type="dcat_dataset_information", _id=identifier, language=language
    )
    return api.get_response()


def get_data_structure(identifier: str, language: str = "fr") -> dict:
    """
    Get the data structure
    Parameters:
        - identifier (str): the dataset's identifier
        - language (str, default='fr'): the language of the response data
            Available are 'fr', 'de', 'it', 'en'.
    Returns:
        - response: datastructure dictionnary
    """
    api = Api(
        api_type="dcat_data_structure", _id=identifier, language=language
    )
    return api.get_response()


def get_nomenclature_path_nodes(
    identifier: str, path: str, filters: dict, language: str = "fr"
) -> dict:
    """
    Get the nodes of a path within a nomenclature
    Parameters:
        - identifier (str): the nomenclature's identifier
        - path (str): the path leading to the nodes
        - filters (dict, default={}): the filters to apply
        - language (str, default='fr'): the language of the response data
            Available are 'fr', 'de', 'it', 'en'.
    Returns:
        - response: dictionnary of the nodes
    """
    api = Api(
        api_type="nomenclature_path_nodes",
        _id=identifier,
        language=language,
        parameters=filters,
    )
    return api.get_response()


def get_nomenclature_one_level(
    identifier: str,
    level_number: int,
    filters: dict,
    language: str = "fr",
    annotations: bool = False,
) -> pd.DataFrame:
    """
    Get one level of a nomenclature
    Parameters:
        - identifier (str): nomenclature's identifier
        - level_number (int): level to export
        - filter (default={}): additionnal filters
        - language (str, default='fr'): response data's language
            Available are 'fr', 'de', 'it', 'en'.
        - annotations (bool, default=False): flag to include annotations
    Returns:
        - response (pd.DataFrame): dataframe with 3 columns
        (Code, Parent and Name in the selected language)
    """
    parameters = (
        f"language={language}"
        f"&level={level_number}"
        f"&annotations={str(annotations).lower()}"
        f"&{stringify_filters(filters)}"
    )
    api = Api(
        api_type="nomenclature_one_level",
        _id=identifier,
        parameters=parameters,
        export_format="CSV",
    )
    return api.get_response()


def get_nomenclature_multiple_levels(
    identifier: str,
    level_from: int,
    level_to: int,
    filters: dict,
    language: str = "fr",
    annotations: bool = False,
) -> pd.DataFrame:
    """
    Get multiple levels of a nomenclature (from `level_from` to `level_to`)
    Parameters:
        - identifier (str): nomenclature's identifier
        - level_from (int): the 1st level to include
        - level_to (int): the last level to include
        - filter (default={}): additionnal filters
        - language (str, default='fr'): response data's language
            Available are 'fr', 'de', 'it', 'en'.
        - annotations (bool, default=False): flag to include annotations
    Returns:
        - response (pd.DataFrame): dataframe columns
        from `level_from` to `level_to` codes
    """
    # Call api with appropriate url and parameters
    parameters = (
        f"language={language}"
        f"&levelFrom={level_from}"
        f"&levelTo={level_to}"
        f"&annotations={str(annotations).lower()}"
        f"&{stringify_filters(filters)}"
    )
    api = Api(
        api_type="nomenclature_multiple_levels",
        _id=identifier,
        parameters=parameters,
        export_format="CSV",
    )
    df = api.get_response()

    # Post-processing:
    # fill sub groups rows with parent group's values (instead of NaN)
    group_columns = [
        *itertools.takewhile(lambda col: col != "Code", df.columns)
    ]
    df[group_columns[0]] = df[group_columns[0]].fillna(method="ffill")
    df[group_columns[1:]] = (
        df.groupby([group_columns[0]])[group_columns[1:]]
    ).apply(lambda x: x.ffill())

    return df


def query_nomenclature(
    identifier: str,
    query: str,
    page_number: int,
    page_size: int,
    filters: dict,
    language: str,
) -> dict:
    """
    Search query within a nomenclature
    Parameters:
        - identifier (str): the nomenclature's identifier
        - query (str): the search query
        - page_number (int): the number of the result page to return
        - page_size (int): the size of each page result
        - filter (default={}): additionnal filters
        - language (str, default='fr'): response data's language
            Available are 'fr', 'de', 'it', 'en'.
    Returns:
        - response (dict): the query result
    """
    parameters = {
        "language": language,
        "query": query,
        "page": page_number,
        "pageSize": page_size,
    }
    parameters = {**parameters, **filters}

    api = Api(
        api_type="nomenclature_search", _id=identifier, parameters=parameters
    )

    return api.get_response()


###############################################################
# dcat APIs
###############################################################


def dcat_list_all_agents():
    """
    List all agents
    Returns:
        - response (dict): all agents
    """
    api = Api(api_type="agents_list", root_url=DCAT_URL)
    return api.get_response()


def dcat_get_agent_from_id(agent_id):
    """
    Get the agent with the corresponding agent id
    Parameters:
        - agent_id (str): agent's id
    Returns:
        - response (dict): agent with this id
    """
    api = Api(api_type="agent_id", _id=agent_id, root_url=DCAT_URL)
    return api.get_response()


def dcat_list_all_datasets():
    """
    List all datasets
    Returns:
        - response (dict): list of all datasets
    """
    api = Api(api_type="dataset_list", root_url=DCAT_URL)
    return api.get_response()


def dcat_get_distributions_from_dataset_id(dataset_id: str):
    """
    Get all distributions for the dataset with the corresponding dataset id
    Parameters:
        - dataset_id (str): dataset's id
    Returns:
        - response (dict): distributions for the dataset with dataset's id
    """
    api = Api(
        api_type="dataset_id_distributions", _id=dataset_id, root_url=DCAT_URL
    )
    return api.get_response()


def dcat_get_dataset_from_id(dataset_id: str):
    """
    Get the dataset with the corresponding id
    Parameters:
        - dataset_id (str): dataset's id
    Returns:
        - response (dict): the dataset
    """
    api = Api(api_type="dataset_id", _id=dataset_id, root_url=DCAT_URL)
    return api.get_response()


def dcat_get_dataset_from_identifier(identifier: str):
    """
    Get the dataset with the corresponding identifier
    Parameters:
        - identidier (str): dataset's identifier
    Returns:
        - response (dict): the dataset
    """
    api = Api(api_type="dataset_identifier", _id=identifier, root_url=DCAT_URL)
    return api.get_response()


def dcat_get_distributions_from_dataset_identifier(identifier: str):
    """
    Get all distributions for the dataset with the corresponding identifier.
    Parameters:
        - identidier (str): dataset's identifier
    Returns:
        - response (dict): all distributions for the dataset with
        the corresponding dataset identifier
    """
    api = Api(
        api_type="dataset_identifier_distributions",
        _id=identifier,
        root_url=DCAT_URL,
    )
    return api.get_response()


def dcat_list_all_distributions():
    """
    List all distributions
    Returns:
        - response (dict): all distributions
    """
    api = Api(api_type="distributions_list", root_url=DCAT_URL)
    return api.get_response()


def dcat_get_distribution_from_id(distribution_id: str):
    """
    Get the distribution with the corresponding id
    Parameters:
        - distribution_id (str): distribution's id
    Returns:
        - response (dict): the distribution
    """
    api = Api(
        api_type="distribution_id", _id=distribution_id, root_url=DCAT_URL
    )
    return api.get_response()
